@@ -63,8 +63,11 @@ init_mteTriggerDeltaTable(void)
 
     /* Register this using the (common) trigger_table_data container */
     rc = netsnmp_tdata_register(reg, trigger_table_data, table_info);
-    if (rc != SNMPERR_SUCCESS)
+    if (rc != SNMPERR_SUCCESS) {
+        snmp_log(LOG_ERR, "%s: netsnmp_tdata_register() returned %d\n",
+                 __func__, rc);
         return;
+    }
     netsnmp_handler_owns_table_info(reg->handler->next);
     DEBUGMSGTL(("disman:event:init", "Trigger Delta Table\n"));
 }
