@@ -59,6 +59,9 @@
 #if HAVE_SYS_DISKLABEL_H
 #define DKTYPENAMES
 #include <sys/disklabel.h>
+#ifndef dragonfly
+#include <sys/disk.h>
+#endif
 #endif
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -78,6 +81,9 @@
 #if HAVE_LIMITS_H
 #include <limits.h>
 #endif
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #ifdef darwin
 #include <CoreFoundation/CoreFoundation.h>
@@ -130,9 +136,6 @@
 	 *
 	 *********************/
 
-void            Init_HR_Disk(void);
-int             Get_Next_HR_Disk(void);
-int             Get_Next_HR_Disk_Partition(char *, size_t, int);
 #if !(defined(aix4) || defined(aix5) || defined(aix6) || defined(aix7))
 static void     Add_HR_Disk_entry(const char *, int, int, int, int,
                                   const char *, int, int);
@@ -176,7 +179,7 @@ static int      HRD_savedCtrl_type;
 static struct hd_driveid HRD_info;
 #endif
 
-#ifdef DIOCGDINFO
+#if defined(DIOCGDINFO) || defined(DIOCGMEDIASIZE)
 static struct disklabel HRD_info;
 #endif
 
@@ -271,6 +274,9 @@ init_hr_disk(void)
 #elif defined(freebsd2)
     Add_HR_Disk_entry("/dev/wd%d%c", -1, -1, 0, 3, "/dev/wd%d", 'a', 'h');
     Add_HR_Disk_entry("/dev/sd%d%c", -1, -1, 0, 3, "/dev/sd%d", 'a', 'h');
+#elif defined(netbsd6)
+    Add_HR_Disk_entry("/dev/wd%d%c", -1, -1, 0, 3, "/dev/rwd%da", 'a', 'h');
+    Add_HR_Disk_entry("/dev/sd%d%c", -1, -1, 0, 3, "/dev/rsd%da", 'a', 'h');
 #elif defined(netbsd1)
     Add_HR_Disk_entry("/dev/wd%d%c", -1, -1, 0, 3, "/dev/wd%dc", 'a', 'h');
     Add_HR_Disk_entry("/dev/sd%d%c", -1, -1, 0, 3, "/dev/sd%dc", 'a', 'h');
@@ -407,7 +413,7 @@ parse_disk_config(const char *token, char *cptr)
                  *p != '\0' && *p != '?' && *p != '*' && *p != '['; p++);
             c = *p;
             *p = '\0';
-            d_str = (char *) malloc(strlen(name) + 1);
+            d_str = strdup(name);
             if (!d_str) {
                 SNMP_FREE(d_new);
                 SNMP_FREE(d_str);
@@ -416,7 +422,6 @@ parse_disk_config(const char *token, char *cptr)
                 config_perror("Out of memory");
                 return;
             }
-            strcpy(d_str, name);
             *p = c;
             di_curr->item_type = ITEM_STRING;
             di_curr->item_details = (void *) d_str;
@@ -810,7 +815,7 @@ Get_Next_HR_Disk(void)
                      */
                     DEBUGMSGTL(("host/hr_disk",
                                 "Get_Next_HR_Disk: %s ignored\n", string));
-                    HRD_history[iindex] = LONG_MAX;
+                    HRD_history[iindex] = (time_t)LONG_MAX;
                     HRD_index++;
                     continue;
                 }
@@ -1054,7 +1059,14 @@ Query_Disk(int fd, const char *devfull)
     }
 #endif
 
-#ifdef DIOCGDINFO
+#if defined(DIOCGMEDIASIZE)
+    unsigned long long size64;
+
+    if (ioctl(fd, DIOCGMEDIASIZE, &size64) < 0)
+	result = -1;
+    HRD_info.d_secperunit = size64 / 512;
+    result = 0;
+#elif defined(DIOCGDINFO)
     result = ioctl(fd, DIOCGDINFO, &HRD_info);
 #endif
 
