@@ -13,14 +13,6 @@
 #include "../ipSystemStatsTable/ipSystemStatsTable.h"
 #include "systemstats_private.h"
 
-#if defined(NETSNMP_IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
-#define _KERNEL 1
-#define _I_DEFINED_KERNEL
-#endif
-#if NETSNMP_IFNET_NEEDS_KERNEL_STRUCTURES
-#define _KERNEL_STRUCTURES
-#endif
-
 #include <sys/types.h>
 #include <dirent.h>
 #include <ctype.h>
@@ -29,7 +21,17 @@
 #include <sys/sysctl.h>
 #include <sys/protosw.h>
 
+#if defined(NETSNMP_IFNET_NEEDS_KERNEL) && !defined(_KERNEL)
+#define _KERNEL 1
+#define _I_DEFINED_KERNEL
+#endif
+#if NETSNMP_IFNET_NEEDS_KERNEL_STRUCTURES
+#define _KERNEL_STRUCTURES
+#endif
 #include <net/if.h>
+#ifdef _I_DEFINED_KERNEL
+#undef _KERNEL
+#endif
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
 #include <netinet/ip.h>
