@@ -179,9 +179,9 @@ static const struct stats_descr ipv4_snmp_stats[] = {
 #define ICMP6_STATS_PREFIX_LEN	5
 #define UDP6_STATS_PREFIX_LEN   4
 
-netsnmp_feature_child_of(linux_ip6_stat_all, libnetsnmpmibs)
+netsnmp_feature_child_of(linux_ip6_stat_all, libnetsnmpmibs);
 
-netsnmp_feature_child_of(linux_read_ip6_stat, linux_ip6_stat_all)
+netsnmp_feature_child_of(linux_read_ip6_stat, linux_ip6_stat_all);
 
 static int
 decode_icmp_msg(char *line, char *data, struct icmp4_msg_mib *msg)
