@@ -100,7 +100,7 @@ int             keepSeconds = 0, peaks = 0;
 int             tableForm = 0;
 int             varbindsPerPacket = 60;
 
-void            processFileArgs(char *fileName);
+static void     processFileArgs(char *fileName);
 
 void
 usage(void)
@@ -398,16 +398,19 @@ main(int argc, char *argv[])
     int             status;
     int             begin, end, last_end;
     int             print = 1;
-    int             exit_code = 0;
+    int             exit_code = 1;
+
+    SOCK_STARTUP;
 
     switch (arg = snmp_parse_args(argc, argv, &session, "C:", &optProc)) {
     case NETSNMP_PARSE_ARGS_ERROR:
-        exit(1);
+        goto out;
     case NETSNMP_PARSE_ARGS_SUCCESS_EXIT:
-        exit(0);
+        exit_code = 0;
+        goto out;
     case NETSNMP_PARSE_ARGS_ERROR_USAGE:
         usage();
-        exit(1);
+        goto out;
     default:
         break;
     }
@@ -418,27 +421,25 @@ main(int argc, char *argv[])
 	if (current_name >= MAX_ARGS) {
 	    fprintf(stderr, "%s: Too many variables specified (max %d)\n",
 	    	argv[optind], MAX_ARGS);
-	    exit(1);
+	    goto out;
 	}
         varinfo[current_name++].name = argv[optind];
     }
 
     if (current_name == 0) {
         usage();
-        exit(1);
+        goto out;
     }
 
     if (dosum) {
 	if (current_name >= MAX_ARGS) {
 	    fprintf(stderr, "Too many variables specified (max %d)\n",
 	    	MAX_ARGS);
-	    exit(1);
+	    goto out;
 	}
         varinfo[current_name++].name = NULL;
     }
 
-    SOCK_STARTUP;
-
     /*
      * open an SNMP session 
      */
@@ -448,8 +449,7 @@ main(int argc, char *argv[])
          * diagnose snmp_open errors with the input netsnmp_session pointer 
          */
         snmp_sess_perror("snmpdelta", &session);
-        SOCK_CLEANUP;
-        exit(1);
+        goto out;
     }
 
     if (tableForm && timestamp) {
@@ -463,8 +463,7 @@ main(int argc, char *argv[])
             if (snmp_parse_oid(vip->name, vip->info_oid, &vip->oidlen) ==
                 NULL) {
                 snmp_perror(vip->name);
-                SOCK_CLEANUP;
-                exit(1);
+                goto close_session;
             }
             sprint_descriptor(vip->descriptor, vip);
             if (tableForm)
@@ -591,7 +590,7 @@ main(int argc, char *argv[])
                         if (vip->type == ASN_COUNTER64) {
                             fprintf(stderr,
                                     "time delta and table form not supported for counter64s\n");
-                            exit(1);
+                            goto close_session;
                         } else {
                             printvalue =
                                 ((float) value * 100) / delta_time;
@@ -742,7 +741,13 @@ main(int argc, char *argv[])
             wait_for_period(period);
         }
     }
+
+    exit_code = 0;
+
+close_session:
     snmp_close(ss);
+
+out:
     SOCK_CLEANUP;
     return (exit_code);
 }
