@@ -23,14 +23,14 @@ extern          "C" {
 #define TRANSPORT_DOMAIN_TCP_IPV6	1,3,6,1,2,1,100,1,6
 NETSNMP_IMPORT oid      netsnmp_TCPIPv6Domain[];
 
-netsnmp_transport *netsnmp_tcp6_transport(struct sockaddr_in6 *addr, 
-					  int local);
+netsnmp_transport *
+netsnmp_tcp6_transport(const struct netsnmp_ep *ep, int local);
 
 /*
  * "Constructor" for transport domain object.  
  */
 
-NETSNMP_IMPORT void     netsnmp_tcpipv6_ctor(void);
+void netsnmp_tcpipv6_ctor(void);
 
 #ifdef __cplusplus
 }
