@@ -1,9 +1,16 @@
 /*
  * container_binary_array.c
- * $Id$
  *
  * see comments in header file.
  *
+ * Portions of this file are subject to the following copyright(s).  See
+ * the Net-SNMP's COPYING file for more details and other copyrights
+ * that may apply:
+ *
+ * Portions of this file are copyrighted by:
+ * Copyright (c) 2016 VMware, Inc. All rights reserved.
+ * Use is subject to license terms specified in the COPYING file
+ * distributed with the Net-SNMP package.
  */
 
 #include <net-snmp/net-snmp-config.h>
@@ -144,17 +151,20 @@ linear_search(const void *val, netsnmp_container *c)
 }
 
 static int
-binary_search(const void *val, netsnmp_container *c, int exact)
+binary_search(const void *val, netsnmp_container *c, int exact, size_t *next)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
     size_t             len = t->count;
     size_t             half;
-    size_t             middle = 0;
     size_t             first = 0;
-    int                result = 0;
+    size_t             middle = 0; /* init not needed; keeps compiler happy */
+    int                result = 0; /* init not needed; keeps compiler happy */
 
-    if (!len)
+    if (!len) {
+        if (NULL != next)
+            *next = 0;
         return -1;
+    }
 
     if (c->flags & CONTAINER_KEY_UNSORTED) {
         if (!exact) {
@@ -170,39 +180,45 @@ binary_search(const void *val, netsnmp_container *c, int exact)
 
     while (len > 0) {
         half = len >> 1;
-        middle = first;
-        middle += half;
-        if ((result =
-             c->compare(t->data[middle], val)) < 0) {
-            first = middle;
-            ++first;
+        middle = first + half;
+        if ((result = c->compare(t->data[middle], val)) < 0) {
+            first = middle + 1;
             len = len - half - 1;
+        } else if (result == 0) {
+            first = middle;
+            break;
         } else {
-            if(result == 0) {
-                first = middle;
-                break;
-            }
             len = half;
         }
     }
 
-    if (first >= t->count)
+    if (first >= t->count) {
+        if (exact && NULL != next)
+            *next = t->count;
         return -1;
+    }
 
-    if(first != middle) {
+    if (first != middle) {
         /* last compare wasn't against first, so get actual result */
         result = c->compare(t->data[first], val);
     }
 
     if(result == 0) {
-        if (!exact) {
-            if (++first == t->count)
-               first = -1;
-        }
-    }
-    else {
-        if(exact)
+        if (exact && NULL != next)
+            *next = first+1;
+        else if (!exact && ++first == t->count) {
+            if (NULL != next)
+                *next = first;
             first = -1;
+        }
+    } else if(exact) {
+        if (NULL != next) {
+            if (result > 0)
+                *next = first;
+            else
+                *next = t->count;
+        }
+        first = -1;
     }
 
     return first;
@@ -240,9 +256,17 @@ netsnmp_binary_array_options_set(netsnmp_container *c, int set, u_int flags)
 #define BA_FLAGS (CONTAINER_KEY_ALLOW_DUPLICATES|CONTAINER_KEY_UNSORTED)
 
     if (set) {
-        if ((flags & BA_FLAGS) == flags)
+        if ((flags & BA_FLAGS) == flags) {
+            /** if turning off unsorted, do sort */
+            int sort = ((c->flags & CONTAINER_KEY_UNSORTED) &&
+                        ! (flags & CONTAINER_KEY_UNSORTED));
             c->flags = flags;
-        else
+            if (sort) {
+                binary_array_table *t = (binary_array_table*)c->container_data;
+                t->dirty = 1; /* force sort */
+                Sort_Array(c);
+            }
+        } else
             flags = (u_int)-1; /* unsupported flag */
     }
     else
@@ -282,7 +306,7 @@ netsnmp_binary_array_get(netsnmp_container *c, const void *key, int exact)
      * if there is a key, search. Otherwise default is 0;
      */
     if (key) {
-        if ((index = binary_search(key, c, exact)) == -1)
+        if ((index = binary_search(key, c, exact, NULL)) == -1)
             return NULL;
         if (!exact &&
             c->flags & CONTAINER_KEY_ALLOW_DUPLICATES) {
@@ -307,6 +331,22 @@ netsnmp_binary_array_get(netsnmp_container *c, const void *key, int exact)
     return t->data[index];
 }
 
+static int
+netsnmp_binary_array_get_at(netsnmp_container *c, size_t pos, void **entry)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+
+    /*
+     * if there is no data, return NULL;
+     */
+    if (!t->count || pos >= t->count || NULL == entry)
+        return -1;
+
+    *entry = t->data[pos];
+
+    return 0;
+}
+
 int
 netsnmp_binary_array_remove_at(netsnmp_container *c, size_t index, void **save)
 {
@@ -319,7 +359,7 @@ netsnmp_binary_array_remove_at(netsnmp_container *c, size_t index, void **save)
      * if there is no data, return NULL;
      */
     if (!t->count)
-        return 0;
+        return -1;
 
     /*
      * find old data and save it, if ptr provided
@@ -343,6 +383,7 @@ netsnmp_binary_array_remove_at(netsnmp_container *c, size_t index, void **save)
 
     return 0;
 }
+
 int
 netsnmp_binary_array_remove(netsnmp_container *c, const void *key, void **save)
 {
@@ -367,7 +408,7 @@ netsnmp_binary_array_remove(netsnmp_container *c, const void *key, void **save)
     /*
      * search
      */
-    if ((index = binary_search(key, c, 1)) == -1)
+    if ((index = binary_search(key, c, 1, NULL)) == -1)
         return -1;
 
     return netsnmp_binary_array_remove_at(c, (size_t)index, save);
@@ -407,61 +448,120 @@ netsnmp_binary_array_clear(netsnmp_container *c,
     ++c->sync;
 }
 
-NETSNMP_STATIC_INLINE int
-netsnmp_binary_array_insert(netsnmp_container *c, const void *entry)
+static int
+_ba_resize_check(binary_array_table *t)
+{
+    size_t new_max;
+    void ** new_data;
+    if (t->max_size > t->count)
+        return 0; /* resize not needed */
+
+    /*
+     * Table is full, so extend it to double the size, or use 10 elements
+     * if it is empty.
+     */
+    new_max = t->max_size > 0 ? 2 * t->max_size : 10;
+    new_data = (void**) realloc(t->data, new_max * sizeof(void*));
+    if (new_data == NULL) {
+        snmp_log(LOG_ERR, "malloc failed in _ba_resize_check\n");
+        return -1; /* error */
+    }
+
+    memset(new_data + t->max_size, 0x0,
+           (new_max - t->max_size) * sizeof(void*));
+
+    t->data = new_data;
+    t->max_size = new_max;
+
+    return 1; /* resized */
+}
+
+static int
+netsnmp_binary_array_insert_before(netsnmp_container *c, size_t index,
+                                   const void *entry, int dirty)
 {
     binary_array_table *t = (binary_array_table*)c->container_data;
-    int             was_dirty = 0;
-    /*
-     * check for duplicates
-     */
-    if (! (c->flags & CONTAINER_KEY_ALLOW_DUPLICATES)) {
-        was_dirty = t->dirty;
-        if (NULL != netsnmp_binary_array_get(c, entry, 1)) {
-            DEBUGMSGTL(("container","not inserting duplicate key\n"));
-            return -1;
-        }
+
+    if (NULL == entry)
+        return -1;
+
+    if (index > (t->count + 1)) {
+        DEBUGMSGTL(("container:insert:before", "index out of range\n"));
+        return -1;
     }
-    
+
+     /*
+      * check if we need to resize the array
+      */
+    _ba_resize_check(t);
+
     /*
-     * check if we need to resize the array
+     * shift array
      */
-    if (t->max_size <= t->count) {
-        /*
-         * Table is full, so extend it to double the size, or use 10 elements
-         * if it is empty.
-         */
-        size_t const new_max = t->max_size > 0 ? 2 * t->max_size : 10;
-        void ** const new_data =
-            (void**) realloc(t->data, new_max * sizeof(void*));
-
-        if (new_data == NULL)
-            return -1;
-
-        memset(new_data + t->max_size, 0x0,
-               (new_max - t->max_size) * sizeof(void*));
-
-        t->data = new_data;
-        t->max_size = new_max;
-    }
+    memmove(&t->data[index+1], &t->data[index],
+            sizeof(void*) * (t->count - index));
 
     /*
      * Insert the new entry into the data array
      */
-    t->data[t->count++] = NETSNMP_REMOVE_CONST(void *, entry);
-    t->dirty = 1;
+    t->data[index] = NETSNMP_REMOVE_CONST(void *, entry);
+    ++t->count;
 
-    /*
-     * if array was dirty before we called get, sync was incremented when
-     * get called SortArray. If we didn't call get or the array wasn't dirty,
-     * bump sync now.
-     */
-    if (!was_dirty)
-        ++c->sync;
+    if (dirty)
+        t->dirty = 1;
+
+    ++c->sync;
 
     return 0;
 }
 
+NETSNMP_STATIC_INLINE int
+netsnmp_binary_array_insert(netsnmp_container *c, const void *const_entry)
+{
+    binary_array_table *t = (binary_array_table*)c->container_data;
+    int             dirty = 0, i = -2;
+    size_t          next, pos;
+    void           *entry = NETSNMP_REMOVE_CONST(void *, const_entry);
+
+    if (NULL == entry)
+        return -1;
+
+    /*
+     * check key if we have at least 1 item and duplicates aren't allowed
+     */
+    if (! (c->flags & CONTAINER_KEY_ALLOW_DUPLICATES) && t->count) {
+        i = binary_search(entry, c, 1, &next);
+        if (i > 0) {
+            DEBUGMSGTL(("container","not inserting duplicate key\n"));
+            return -1;
+        }
+    }
+ 
+    /*
+     * if unsorted, just add at the end
+     */
+    if (c->flags & CONTAINER_KEY_UNSORTED) {
+        pos = t->count;
+        dirty = 1;
+    } else {
+        /** if we haven't searched for key yet, do it now */
+        if (-2 == i) {
+            if (0 == t->count) {
+                next = 0;
+                i = -1;
+            } else
+                i = binary_search(entry, c, 1, &next);
+        }
+
+        pos = next;
+        if ( i > 0 )  /* if key found, advance past any dups */
+            while (pos < t->count && c->compare(t->data[pos], entry) == 0)
+                ++pos;
+    }
+
+    return netsnmp_binary_array_insert_before(c, pos, entry, dirty);
+}
+
 /**********************************************************************
  *
  * Special case support for subsets
@@ -485,11 +585,9 @@ binary_search_for_start(netsnmp_index *val, netsnmp_container *c)
 
     while (len > 0) {
         half = len >> 1;
-        middle = first;
-        middle += half;
+        middle = first + half;
         if ((result = c->ncompare(t->data[middle], val)) < 0) {
-            first = middle;
-            ++first;
+            first = middle + 1;
             len = len - half - 1;
         } else
             len = half;
@@ -513,12 +611,12 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
     /*
      * if there is no data, return NULL;
      */
-    if (!c || !key)
+    if (!c || !key || !len)
         return NULL;
 
     t = (binary_array_table*)c->container_data;
     netsnmp_assert(c->ncompare);
-    if (!t->count | !c->ncompare)
+    if (!t->count || !c->ncompare)
         return NULL;
 
     /*
@@ -541,6 +639,9 @@ netsnmp_binary_array_get_subset(netsnmp_container *c, void *key, int *len)
     }
 
     *len = end - start + 1;
+    if (*len <= 0)
+        return NULL;
+
     subset = (void **)malloc((*len) * sizeof(void*));
     if (subset)
         memcpy(subset, &t->data[start], sizeof(void*) * (*len));
@@ -571,6 +672,13 @@ _ba_insert(netsnmp_container *container, const void *data)
     return netsnmp_binary_array_insert(container, data);
 }
 
+static int
+_ba_insert_before(netsnmp_container *container, size_t index, void *data)
+{
+    /** don't trust users direct-acces inserts, mark array dirty */
+    return netsnmp_binary_array_insert_before(container, index, data, 1);
+}
+
 static int
 _ba_remove(netsnmp_container *container, const void *data)
 {
@@ -612,13 +720,12 @@ _ba_get_subset(netsnmp_container *container, void *data)
     int len;
 
     rtn = netsnmp_binary_array_get_subset(container, data, &len);
-    if ((NULL==rtn) || (len <=0))
+    if (NULL==rtn)
         return NULL;
     
     va = SNMP_MALLOC_TYPEDEF(netsnmp_void_array);
-    if (NULL==va)
-    {
-        free (rtn);
+    if (va == NULL) {
+        free(rtn);
         return NULL;
     }
     
@@ -708,7 +815,10 @@ netsnmp_container_get_binary_array(void)
     c->clear = _ba_clear;
     c->options = _ba_options;
     c->duplicate = _ba_duplicate;
-        
+    c->get_at = netsnmp_binary_array_get_at;
+    c->remove_at = netsnmp_binary_array_remove_at;
+    c->insert_before = _ba_insert_before;
+
     return c;
 }
 
