@@ -131,7 +131,7 @@ int MD5_hmac(const u_char * data, size_t len, u_char * mac, size_t maclen,
              const u_char * secret, size_t secretlen);
 #endif
 
-static netsnmp_auth_alg_info _auth_alg_info[] = {
+static const netsnmp_auth_alg_info _auth_alg_info[] = {
     { NETSNMP_USMAUTH_NOAUTH, "usmNoAuthProtocol", usmNoAuthProtocol,
       OID_LENGTH(usmNoAuthProtocol), 0, 0 },
     { NETSNMP_USMAUTH_HMACSHA1, "usmHMACSHA1AuthProtocol",
@@ -161,7 +161,7 @@ static netsnmp_auth_alg_info _auth_alg_info[] = {
     { -1, "unknown", NULL, 0, 0, 0 }
 };
 
-static netsnmp_priv_alg_info _priv_alg_info[] = {
+static const netsnmp_priv_alg_info _priv_alg_info[] = {
     { USM_CREATE_USER_PRIV_NONE, "usmNoPrivProtocol",
       usmNoPrivProtocol, OID_LENGTH(usmNoPrivProtocol), 0, 0, 0 },
 #ifndef NETSNMP_DISABLE_DES
@@ -209,7 +209,7 @@ static netsnmp_priv_alg_info _priv_alg_info[] = {
  *
  * returns a pointer to a netsnmp_priv_alg_info struct
  */
-netsnmp_priv_alg_info *
+const netsnmp_priv_alg_info *
 sc_get_priv_alg_byoid(const oid *privoid, u_int len)
 {
     int i = 0;
@@ -239,7 +239,7 @@ sc_get_priv_alg_byoid(const oid *privoid, u_int len)
  *
  * returns a pointer to a netsnmp_priv_alg_info struct
  */
-netsnmp_priv_alg_info *
+const netsnmp_priv_alg_info *
 sc_get_priv_alg_bytype(u_int type)
 {
     int i = 0;
@@ -260,7 +260,7 @@ sc_get_priv_alg_bytype(u_int type)
  *
  * returns a pointer to a netsnmp_auth_alg_info struct
  */
-netsnmp_auth_alg_info *
+const netsnmp_auth_alg_info *
 sc_find_auth_alg_byoid(const oid *authoid, u_int len)
 {
     int i = 0;
@@ -290,7 +290,7 @@ sc_find_auth_alg_byoid(const oid *authoid, u_int len)
  *
  * returns a pointer to a netsnmp_auth_alg_info struct
  */
-netsnmp_auth_alg_info *
+const netsnmp_auth_alg_info *
 sc_get_auth_alg_byindex(u_int index)
 {
     DEBUGTRACE;
@@ -307,7 +307,7 @@ sc_get_auth_alg_byindex(u_int index)
  *
  * returns a pointer to a netsnmp_auth_alg_info struct
  */
-netsnmp_auth_alg_info *
+const netsnmp_auth_alg_info *
 sc_find_auth_alg_bytype(u_int type)
 {
     int i = 0;
@@ -335,7 +335,7 @@ sc_find_auth_alg_bytype(u_int type)
 int
 sc_get_authtype(const oid * hashtype, u_int hashtype_len)
 {
-    netsnmp_auth_alg_info *aai;
+    const netsnmp_auth_alg_info *aai;
 
     DEBUGTRACE;
 
@@ -349,7 +349,7 @@ sc_get_authtype(const oid * hashtype, u_int hashtype_len)
 int
 sc_get_privtype(const oid * privtype, u_int privtype_len)
 {
-    netsnmp_priv_alg_info *pai;
+    const netsnmp_priv_alg_info *pai;
 
     DEBUGTRACE;
 
@@ -371,7 +371,7 @@ sc_get_privtype(const oid * privtype, u_int privtype_len)
 int
 sc_get_auth_maclen(int hashtype)
 {
-    netsnmp_auth_alg_info *aai;
+    const netsnmp_auth_alg_info *aai;
 
     DEBUGTRACE;
 
@@ -392,7 +392,7 @@ sc_get_auth_maclen(int hashtype)
 int
 sc_get_proper_auth_length_bytype(int hashtype)
 {
-    netsnmp_auth_alg_info *aai;
+    const netsnmp_auth_alg_info *aai;
 
     DEBUGTRACE;
 
@@ -411,7 +411,7 @@ sc_get_proper_auth_length_bytype(int hashtype)
 oid *
 sc_get_auth_oid(int type, size_t *oid_len)
 {
-    netsnmp_auth_alg_info *ai;
+    const netsnmp_auth_alg_info *ai;
 
     DEBUGTRACE;
 
@@ -433,7 +433,7 @@ sc_get_auth_oid(int type, size_t *oid_len)
 const char*
 sc_get_auth_name(int type)
 {
-    netsnmp_auth_alg_info *ai;
+    const netsnmp_auth_alg_info *ai;
 
     DEBUGTRACE;
 
@@ -452,7 +452,7 @@ sc_get_auth_name(int type)
 oid *
 sc_get_priv_oid(int type, size_t *oid_len)
 {
-    netsnmp_priv_alg_info *ai;
+    const netsnmp_priv_alg_info *ai;
 
     DEBUGTRACE;
 
@@ -490,7 +490,7 @@ netsnmp_feature_child_of(scapi_get_proper_priv_length, netsnmp_unused);
 int
 sc_get_proper_priv_length(const oid * privtype, u_int privtype_len)
 {
-    netsnmp_priv_alg_info *pai;
+    const netsnmp_priv_alg_info *pai;
 
     DEBUGTRACE;
 
@@ -507,7 +507,7 @@ sc_get_proper_priv_length(const oid * privtype, u_int privtype_len)
  *
  * returns a pointer to a netsnmp_priv_alg_info struct
  */
-netsnmp_priv_alg_info *
+const netsnmp_priv_alg_info *
 sc_get_priv_alg_byindex(u_int index)
 {
     DEBUGTRACE;
@@ -522,7 +522,7 @@ sc_get_priv_alg_byindex(u_int index)
 int
 sc_get_proper_priv_length_bytype(int privtype)
 {
-    netsnmp_priv_alg_info *pai;
+    const netsnmp_priv_alg_info *pai;
 
     DEBUGTRACE;
 
@@ -1175,7 +1175,7 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
     int             rval = SNMPERR_SUCCESS;
     u_char          pad_block[128];      /* bigger than anything I need */
     u_char          my_iv[128];  /* ditto */
-    netsnmp_priv_alg_info *pai = NULL;
+    const netsnmp_priv_alg_info *pai = NULL;
 #ifndef NETSNMP_DISABLE_DES
     int             pad, plast, pad_size = 0;
 #ifdef OLD_DES
@@ -1370,7 +1370,7 @@ sc_encrypt(const oid * privtype, size_t privtypelen,
 {
     int             rval = SNMPERR_SUCCESS, priv_type
     u_char	    pkcs_des_key[8];
-    netsnmp_priv_alg_info *pai;
+    const netsnmp_priv_alg_info *pai;
 
     DEBUGTRACE;
 
@@ -1469,7 +1469,7 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
 #endif
     DES_cblock      key_struct;
 #endif
-    netsnmp_priv_alg_info *pai = NULL;
+    const netsnmp_priv_alg_info *pai = NULL;
 
     DEBUGTRACE;
 
@@ -1584,7 +1584,7 @@ sc_decrypt(const oid * privtype, size_t privtypelen,
 {
     int             rval = SNMPERR_SUCCESS;
     u_char	    pkcs_des_key[8];
-    netsnmp_priv_alg_info *pai;
+    const netsnmp_priv_alg_info *pai;
 
     DEBUGTRACE;
 
