@@ -196,6 +196,12 @@ elif test -x /cygdrive/c/windows/system32/netstat ; then
 elif test -x /c/Windows/System32/netstat ; then
     # MinGW + MSYS
     NETSTAT=/c/Windows/System32/netstat
+elif test -x /usr/sbin/ss ; then
+    # Fedora24, RHEL7 does not install netstat as standard
+    NETSTAT=/usr/sbin/ss
+elif test -x /usr/bin/ss ; then
+    # Debian 10 (buster) put ss in /usr/bin/ss
+    NETSTAT=/usr/bin/ss
 else
     NETSTAT=""
 fi
@@ -211,7 +217,7 @@ fi
 
 PROBE_FOR_PORT() {
     BASE_PORT=$1
-    MAX_RETRIES=10
+    MAX_RETRIES=30
     if test -x "$NETSTAT" ; then
         if test -z "$RANDOM"; then
             RANDOM=2
